import {
  defineComponent,
  onMounted,
  PropType,
  reactive,
  ref,
  VNode,
  watch,
} from 'vue';
import { useRoute } from 'vue-router';
import QueryForm, { Schema } from '@/components/QueryForm';
import keys from 'lodash/keys';
import isEmpty from 'lodash/isEmpty';
import './index.scss';
import { pickBy } from 'lodash';

export interface Column<T extends any> {
  label?: string;
  width?: string | number;
  prop?: keyof T;
  rules?: { type?: string; message: string; required?: boolean }[];
  required?: boolean;
  render?: (value: T[keyof T], row: T, index: number) => VNode;
}

export interface Data {
  data: any[];
  pageSize: number;
  total: number;
  pageNum: number;
}

export default defineComponent({
  name: 'Container',
  props: {
    loading: Boolean,
    refeshKey: String as PropType<string | null>,
    // form props. copy from QueryForm
    schema: Array as PropType<Schema[]>,
    form: { type: Object, required: true },
    suffix: {
      type: [Object, String] as PropType<JSX.Element | string>,
      default: undefined,
    },
    onReset: Function,
    onQuery: Function as PropType<(params: any) => void>,
    // table
    data: Object,
    columns: Array as PropType<Column<any>[]>,
    params: Object,
    transformQueryParams: {
      type: Function,
      default: (params: object) => pickBy(params, (p) => !isEmpty(p)),
    },
  },
  setup(props) {
    const state = reactive({
      tableData: [] as any[],
      loading: !!props.loading,
    });

    const route = useRoute();
    const params = ref(route.params);
    const form = ref(props.form);

    const handleQueryData = (form: object) => {
      props.onQuery && props.onQuery({ ...form });
    };
    const handleSubmit = (form: object) => {
      handleQueryData({ ...params.value, ...props.transformQueryParams(form) });
    };

    const handlePageSizeChange = (pageSize: number) => {
      handleQueryData({
        ...params.value,
        ...props.transformQueryParams(props.form),
        pageSize,
        pageNum: 1,
      });
    };

    const handlePageNumChange = (pageNum: number) => {
      handleQueryData({
        ...params.value,
        ...props.transformQueryParams(props.form),
        pageSize: props.data?.pageSize ?? 10,
        pageNum,
      });
    };

    onMounted(() => {
      handleQueryData({
        ...params.value,
        ...props.transformQueryParams(props.form),
      });
    });

    watch(
      () => props.refeshKey,
      (next) => {
        if (next)
          handleQueryData({
            ...params.value,
            ...props.transformQueryParams(props.form),
          });
      },
    );

    watch(
      () => route.params,
      (nextParams, preParams) => {
        if (
          isEmpty(nextParams) ||
          JSON.stringify(nextParams) === JSON.stringify(preParams) ||
          Object.keys(nextParams).toString() !==
            Object.keys(preParams).toString()
        )
          return;

        const resetForm = keys(props.form).reduce(
          (c, n) => ({ ...c, [n]: undefined }),
          {},
        );
        form.value = resetForm;
        params.value = nextParams;
        handleQueryData({
          ...nextParams,
          ...resetForm,
          pageSize: 10,
          pageNum: 1,
        });
      },
    );

    return () => (
      <div v-loading={props.loading} class="table-form-container">
        <QueryForm
          schema={props.schema}
          form={form.value}
          suffix={props.suffix}
          onSubmit={handleSubmit}
        />

        <el-table
          data={props.data?.rows ?? []}
          height={'calc(100vh - 60px - 88px - 124px)'}>
          {props.columns &&
            props.columns.map((col, key) => (
              <el-table-column
                prop={col.prop}
                label={col.label}
                min-width={col.width}
                v-slots={
                  col.render && {
                    default: (slotProps: any) =>
                      col.render &&
                      col.render(
                        slotProps?.row?.[col.prop as string],
                        slotProps?.row,
                        slotProps.$index,
                      ),
                  }
                }
              />
            ))}
        </el-table>
        <div class="pagination-content">
          <el-pagination
            hide-on-single-page={state.tableData?.length === 0}
            background
            page-sizes={[10, 20, 30, 40]}
            page-size={props.data?.pageSize ?? 10}
            total={props.data?.total ?? 0}
            currentPage={props.data?.pageNum ?? 1}
            layout="prev, pager, next,sizes, jumper"
            onSizeChange={handlePageSizeChange}
            onCurrentChange={handlePageNumChange}
          />
        </div>
      </div>
    );
  },
});
